@@ -33,10 +33,8 @@ import org.apache.tomcat.util.log.UserDataHelper;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
- * The legacy (up to early Tomcat 8 releases) cookie parser based on RFC6265,
- * RFC2109 and RFC2616.
- *
- * This class is not thread-safe.
+ * The legacy (up to early Tomcat 8 releases) cookie parser based on RFC6265, RFC2109 and RFC2616. This class is not
+ * thread-safe.
  *
  * @author Costin Manolache
  * @author kevin seguin
@@ -47,17 +45,15 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
 
     private static final UserDataHelper userDataLog = new UserDataHelper(log);
 
-    private static final StringManager sm =
-            StringManager.getManager("org.apache.tomcat.util.http");
+    private static final StringManager sm = StringManager.getManager("org.apache.tomcat.util.http");
 
-    private static final char[] V0_SEPARATORS = {',', ';', ' ', '\t'};
+    private static final char[] V0_SEPARATORS = { ',', ';', ' ', '\t' };
     private static final BitSet V0_SEPARATOR_FLAGS = new BitSet(128);
 
     // Excludes '/' since configuration controls whether or not to treat '/' as
     // a separator
-    private static final char[] HTTP_SEPARATORS = new char[] {
-            '\t', ' ', '\"', '(', ')', ',', ':', ';', '<', '=', '>', '?', '@',
-            '[', '\\', ']', '{', '}' };
+    private static final char[] HTTP_SEPARATORS = new char[] { '\t', ' ', '\"', '(', ')', ',', ':', ';', '<', '=', '>',
+            '?', '@', '[', '\\', ']', '{', '}' };
 
     static {
         for (char c : V0_SEPARATORS) {
@@ -65,8 +61,8 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
         }
     }
 
-    private final boolean STRICT_SERVLET_COMPLIANCE =
-            Boolean.getBoolean("org.apache.catalina.STRICT_SERVLET_COMPLIANCE");
+    private final boolean STRICT_SERVLET_COMPLIANCE = Boolean
+            .getBoolean("org.apache.catalina.STRICT_SERVLET_COMPLIANCE");
 
     private boolean allowEqualsInValue = false;
 
@@ -107,11 +103,10 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
         }
 
         /*
-         * Some browsers (e.g. IE6 and IE7) do not handle quoted Path values even
-         * when Version is set to 1. To allow for this, we support a property
-         * FWD_SLASH_IS_SEPARATOR which, when false, means a '/' character will not
-         * be treated as a separator, potentially avoiding quoting and the ensuing
-         * side effect of having the cookie upgraded to version 1.
+         * Some browsers (e.g. IE6 and IE7) do not handle quoted Path values even when Version is set to 1. To allow for
+         * this, we support a property FWD_SLASH_IS_SEPARATOR which, when false, means a '/' character will not be
+         * treated as a separator, potentially avoiding quoting and the ensuing side effect of having the cookie
+         * upgraded to version 1.
          *
          * For now, we apply this rule globally rather than just to the Path attribute.
          */
@@ -214,8 +209,8 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
         while (pos >= 0) {
             MessageBytes cookieValue = headers.getValue(pos);
 
-            if (cookieValue != null && !cookieValue.isNull() ) {
-                if (cookieValue.getType() != MessageBytes.T_BYTES ) {
+            if (cookieValue != null && !cookieValue.isNull()) {
+                if (cookieValue.getType() != MessageBytes.T_BYTES) {
                     Exception e = new Exception();
                     // TODO: Review this in light of HTTP/2
                     log.debug("Cookies: Parsing cookie as String. Expected bytes.", e);
@@ -244,13 +239,11 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
     public String generateHeader(Cookie cookie, HttpServletRequest request) {
 
         /*
-         * The spec allows some latitude on when to send the version attribute
-         * with a Set-Cookie header. To be nice to clients, we'll make sure the
-         * version attribute is first. That means checking the various things
-         * that can cause us to switch to a v1 cookie first.
+         * The spec allows some latitude on when to send the version attribute with a Set-Cookie header. To be nice to
+         * clients, we'll make sure the version attribute is first. That means checking the various things that can
+         * cause us to switch to a v1 cookie first.
          *
-         * Note that by checking for tokens we will also throw an exception if a
-         * control character is encountered.
+         * Note that by checking for tokens we will also throw an exception if a control character is encountered.
          */
         int version = cookie.getVersion();
         String value = cookie.getValue();
@@ -278,11 +271,11 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
         // Add version 1 specific information
         if (version == 1) {
             // Version=1 ... required
-            buf.append ("; Version=1");
+            buf.append("; Version=1");
 
             // Comment=comment
             if (comment != null) {
-                buf.append ("; Comment=");
+                buf.append("; Comment=");
                 maybeQuote(buf, comment, version);
             }
         }
@@ -297,35 +290,33 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
         int maxAge = cookie.getMaxAge();
         if (maxAge >= 0) {
             if (version > 0) {
-                buf.append ("; Max-Age=");
-                buf.append (maxAge);
+                buf.append("; Max-Age=");
+                buf.append(maxAge);
             }
             // IE6, IE7 and possibly other browsers don't understand Max-Age.
             // They do understand Expires, even with V1 cookies!
             if (version == 0 || getAlwaysAddExpires()) {
                 // Wdy, DD-Mon-YY HH:MM:SS GMT ( Expires Netscape format )
-                buf.append ("; Expires=");
+                buf.append("; Expires=");
                 // To expire immediately we need to set the time in past
                 if (maxAge == 0) {
-                    buf.append( ANCIENT_DATE );
+                    buf.append(ANCIENT_DATE);
                 } else {
-                    COOKIE_DATE_FORMAT.get().format(
-                            new Date(System.currentTimeMillis() + maxAge * 1000L),
-                            buf,
+                    COOKIE_DATE_FORMAT.get().format(new Date(System.currentTimeMillis() + maxAge * 1000L), buf,
                             new FieldPosition(0));
                 }
             }
         }
 
         // Path=path
-        if (path!=null) {
-            buf.append ("; Path=");
+        if (path != null) {
+            buf.append("; Path=");
             maybeQuote(buf, path, version);
         }
 
         // Secure
         if (cookie.getSecure()) {
-          buf.append ("; Secure");
+            buf.append("; Secure");
         }
 
         // HttpOnly
@@ -349,11 +340,11 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
             buf.append("\"\"");
         } else if (alreadyQuoted(value)) {
             buf.append('"');
-            escapeDoubleQuotes(buf, value,1,value.length()-1);
+            escapeDoubleQuotes(buf, value, 1, value.length() - 1);
             buf.append('"');
         } else if (needsQuotes(value, version)) {
             buf.append('"');
-            escapeDoubleQuotes(buf, value,0,value.length());
+            escapeDoubleQuotes(buf, value, 0, value.length());
             buf.append('"');
         } else {
             buf.append(value);
@@ -369,7 +360,7 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
 
         for (int i = beginIndex; i < endIndex; i++) {
             char c = s.charAt(i);
-            if (c == '\\' ) {
+            if (c == '\\') {
                 b.append('\\').append('\\');
             } else if (c == '"') {
                 b.append('\\').append('"');
@@ -396,11 +387,9 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
         for (; i < len; i++) {
             char c = value.charAt(i);
             if ((c < 0x20 && c != '\t') || c >= 0x7f) {
-                throw new IllegalArgumentException(
-                        "Control character in cookie value or attribute.");
+                throw new IllegalArgumentException("Control character in cookie value or attribute.");
             }
-            if (version == 0 && !allowedWithoutQuotes.get(c) ||
-                    version == 1 && isHttpSeparator(c)) {
+            if (version == 0 && !allowedWithoutQuotes.get(c) || version == 1 && isHttpSeparator(c)) {
                 return true;
             }
         }
@@ -408,21 +397,15 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
     }
 
 
-    private static boolean alreadyQuoted (String value) {
-        return value.length() >= 2 &&
-                value.charAt(0) == '\"' &&
-                value.charAt(value.length() - 1) == '\"';
+    private static boolean alreadyQuoted(String value) {
+        return value.length() >= 2 && value.charAt(0) == '\"' && value.charAt(value.length() - 1) == '\"';
     }
 
 
     /**
-     * Parses a cookie header after the initial "Cookie:"
-     * [WS][$]token[WS]=[WS](token|QV)[;|,]
-     * RFC 2965 / RFC 2109
-     * JVK
+     * Parses a cookie header after the initial "Cookie:" [WS][$]token[WS]=[WS](token|QV)[;|,] RFC 2965 / RFC 2109 JVK
      */
-    private void processCookieHeader(byte bytes[], int off, int len,
-            ServerCookies serverCookies) {
+    private void processCookieHeader(byte bytes[], int off, int len, ServerCookies serverCookies) {
 
         if (len <= 0 || bytes == null) {
             return;
@@ -443,12 +426,10 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
             isQuoted = false;
 
             // Skip whitespace and non-token characters (separators)
-            while (pos < end &&
-                   (isHttpSeparator((char) bytes[pos]) &&
-                           !getAllowHttpSepsInV0() ||
-                    isV0Separator((char) bytes[pos]) ||
-                    isWhiteSpace(bytes[pos])))
-                {pos++; }
+            while (pos < end && (isHttpSeparator((char) bytes[pos]) && !getAllowHttpSepsInV0() ||
+                    isV0Separator((char) bytes[pos]) || isWhiteSpace(bytes[pos]))) {
+                pos++;
+            }
 
             if (pos >= end) {
                 return;
@@ -462,10 +443,12 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
 
             // Get the cookie/attribute name. This must be a token
             valueEnd = valueStart = nameStart = pos;
-            pos = nameEnd = getTokenEndPosition(bytes,pos,end,version,true);
+            pos = nameEnd = getTokenEndPosition(bytes, pos, end, version, true);
 
             // Skip whitespace
-            while (pos < end && isWhiteSpace(bytes[pos])) {pos++; }
+            while (pos < end && isWhiteSpace(bytes[pos])) {
+                pos++;
+            }
 
 
             // Check for an '=' -- This could also be a name-only
@@ -486,79 +469,74 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
                 // Determine what type of value this is, quoted value,
                 // token, name-only with an '=', or other (bad)
                 switch (bytes[pos]) {
-                case '"': // Quoted Value
-                    isQuoted = true;
-                    valueStart = pos + 1; // strip "
-                    // getQuotedValue returns the position before
-                    // at the last quote. This must be dealt with
-                    // when the bytes are copied into the cookie
-                    valueEnd = getQuotedValueEndPosition(bytes, valueStart, end);
-                    // We need pos to advance
-                    pos = valueEnd;
-                    // Handles cases where the quoted value is
-                    // unterminated and at the end of the header,
-                    // e.g. [myname="value]
-                    if (pos >= end) {
-                        return;
-                    }
-                    break;
-                case ';':
-                case ',':
-                    // Name-only cookie with an '=' after the name token
-                    // This may not be RFC compliant
-                    valueStart = valueEnd = -1;
-                    // The position is OK (On a delimiter)
-                    break;
-                default:
-                    if (version == 0 &&
-                                !isV0Separator((char)bytes[pos]) &&
-                                getAllowHttpSepsInV0() ||
-                            !isHttpSeparator((char)bytes[pos]) ||
-                            bytes[pos] == '=') {
-                        // Token
-                        valueStart = pos;
-                        // getToken returns the position at the delimiter
-                        // or other non-token character
-                        valueEnd = getTokenEndPosition(bytes, valueStart, end, version, false);
+                    case '"': // Quoted Value
+                        isQuoted = true;
+                        valueStart = pos + 1; // strip "
+                        // getQuotedValue returns the position before
+                        // at the last quote. This must be dealt with
+                        // when the bytes are copied into the cookie
+                        valueEnd = getQuotedValueEndPosition(bytes, valueStart, end);
                         // We need pos to advance
                         pos = valueEnd;
-                        // Edge case. If value starts with '=' but this is not
-                        // allowed in a value make sure we treat this as no
-                        // value being present
-                        if (valueStart == valueEnd) {
-                            valueStart = -1;
-                            valueEnd = -1;
+                        // Handles cases where the quoted value is
+                        // unterminated and at the end of the header,
+                        // e.g. [myname="value]
+                        if (pos >= end) {
+                            return;
                         }
-                    } else  {
-                        // INVALID COOKIE, advance to next delimiter
-                        // The starting character of the cookie value was
-                        // not valid.
-                        UserDataHelper.Mode logMode = userDataLog.getNextMode();
-                        if (logMode != null) {
-                            String message = sm.getString(
-                                    "cookies.invalidCookieToken");
-                            switch (logMode) {
-                                case INFO_THEN_DEBUG:
-                                    message += sm.getString(
-                                            "cookies.fallToDebug");
-                                    //$FALL-THROUGH$
-                                case INFO:
-                                    log.info(message);
-                                    break;
-                                case DEBUG:
-                                    log.debug(message);
+                        break;
+                    case ';':
+                    case ',':
+                        // Name-only cookie with an '=' after the name token
+                        // This may not be RFC compliant
+                        valueStart = valueEnd = -1;
+                        // The position is OK (On a delimiter)
+                        break;
+                    default:
+                        if (version == 0 && !isV0Separator((char) bytes[pos]) && getAllowHttpSepsInV0() ||
+                                !isHttpSeparator((char) bytes[pos]) || bytes[pos] == '=') {
+                            // Token
+                            valueStart = pos;
+                            // getToken returns the position at the delimiter
+                            // or other non-token character
+                            valueEnd = getTokenEndPosition(bytes, valueStart, end, version, false);
+                            // We need pos to advance
+                            pos = valueEnd;
+                            // Edge case. If value starts with '=' but this is not
+                            // allowed in a value make sure we treat this as no
+                            // value being present
+                            if (valueStart == valueEnd) {
+                                valueStart = -1;
+                                valueEnd = -1;
                             }
+                        } else {
+                            // INVALID COOKIE, advance to next delimiter
+                            // The starting character of the cookie value was
+                            // not valid.
+                            UserDataHelper.Mode logMode = userDataLog.getNextMode();
+                            if (logMode != null) {
+                                String message = sm.getString("cookies.invalidCookieToken");
+                                switch (logMode) {
+                                    case INFO_THEN_DEBUG:
+                                        message += sm.getString("cookies.fallToDebug");
+                                        //$FALL-THROUGH$
+                                    case INFO:
+                                        log.info(message);
+                                        break;
+                                    case DEBUG:
+                                        log.debug(message);
+                                }
+                            }
+                            while (pos < end && bytes[pos] != ';' && bytes[pos] != ',') {
+                                pos++;
+                            }
+                            pos++;
+                            // Make sure no special avpairs can be attributed to
+                            // the previous cookie by setting the current cookie
+                            // to null
+                            sc = null;
+                            continue;
                         }
-                        while (pos < end && bytes[pos] != ';' &&
-                               bytes[pos] != ',')
-                            {pos++; }
-                        pos++;
-                        // Make sure no special avpairs can be attributed to
-                        // the previous cookie by setting the current cookie
-                        // to null
-                        sc = null;
-                        continue;
-                    }
                 }
             } else {
                 // Name only cookie
@@ -572,7 +550,9 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
             // in a good state.
 
             // Skip whitespace
-            while (pos < end && isWhiteSpace(bytes[pos])) {pos++; }
+            while (pos < end && isWhiteSpace(bytes[pos])) {
+                pos++;
+            }
 
 
             // Make sure that after the cookie we have a separator. This
@@ -589,11 +569,10 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
                 isSpecial = false;
                 // $Version must be the first avpair in the cookie header
                 // (sc must be null)
-                if (equals( "Version", bytes, nameStart, nameEnd) &&
-                    sc == null) {
+                if (equals("Version", bytes, nameStart, nameEnd) && sc == null) {
                     // Set version
-                    if( bytes[valueStart] =='1' && valueEnd == (valueStart+1)) {
-                        version=1;
+                    if (bytes[valueStart] == '1' && valueEnd == (valueStart + 1)) {
+                        version = 1;
                     } else {
                         // unknown version (Versioning is not very strict)
                     }
@@ -606,25 +585,21 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
                 }
 
                 // Domain is more common, so it goes first
-                if (equals( "Domain", bytes, nameStart, nameEnd)) {
-                    sc.getDomain().setBytes( bytes,
-                                           valueStart,
-                                           valueEnd-valueStart);
+                if (equals("Domain", bytes, nameStart, nameEnd)) {
+                    sc.getDomain().setBytes(bytes, valueStart, valueEnd - valueStart);
                     continue;
                 }
 
-                if (equals( "Path", bytes, nameStart, nameEnd)) {
-                    sc.getPath().setBytes( bytes,
-                                           valueStart,
-                                           valueEnd-valueStart);
+                if (equals("Path", bytes, nameStart, nameEnd)) {
+                    sc.getPath().setBytes(bytes, valueStart, valueEnd - valueStart);
                     continue;
                 }
 
                 // v2 cookie attributes - skip them
-                if (equals( "Port", bytes, nameStart, nameEnd)) {
+                if (equals("Port", bytes, nameStart, nameEnd)) {
                     continue;
                 }
-                if (equals( "CommentURL", bytes, nameStart, nameEnd)) {
+                if (equals("CommentURL", bytes, nameStart, nameEnd)) {
                     continue;
                 }
 
@@ -650,13 +625,11 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
                 }
 
                 sc = serverCookies.addCookie();
-                sc.setVersion( version );
-                sc.getName().setBytes( bytes, nameStart,
-                                       nameEnd-nameStart);
+                sc.setVersion(version);
+                sc.getName().setBytes(bytes, nameStart, nameEnd - nameStart);
 
                 if (valueStart != -1) { // Normal AVPair
-                    sc.getValue().setBytes( bytes, valueStart,
-                            valueEnd-valueStart);
+                    sc.getValue().setBytes(bytes, valueStart, valueEnd - valueStart);
                     if (isQuoted) {
                         // We know this is a byte value so this is safe
                         unescapeDoubleQuotes(sc.getValue().getByteChunk());
@@ -672,18 +645,14 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
 
 
     /**
-     * Given the starting position of a token, this gets the end of the
-     * token, with no separator characters in between.
+     * Given the starting position of a token, this gets the end of the token, with no separator characters in between.
      * JVK
      */
-    private int getTokenEndPosition(byte bytes[], int off, int end,
-            int version, boolean isName){
+    private int getTokenEndPosition(byte bytes[], int off, int end, int version, boolean isName) {
         int pos = off;
-        while (pos < end &&
-                (!isHttpSeparator((char)bytes[pos]) ||
-                 version == 0 && getAllowHttpSepsInV0() && bytes[pos] != '=' &&
-                        !isV0Separator((char)bytes[pos]) ||
-                 !isName && bytes[pos] == '=' && getAllowEqualsInValue())) {
+        while (pos < end && (!isHttpSeparator((char) bytes[pos]) ||
+                version == 0 && getAllowHttpSepsInV0() && bytes[pos] != '=' && !isV0Separator((char) bytes[pos]) ||
+                !isName && bytes[pos] == '=' && getAllowEqualsInValue())) {
             pos++;
         }
 
@@ -697,8 +666,7 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
     private boolean isHttpSeparator(final char c) {
         if (c < 0x20 || c >= 0x7f) {
             if (c != 0x09) {
-                throw new IllegalArgumentException(
-                        "Control character in cookie value or attribute.");
+                throw new IllegalArgumentException("Control character in cookie value or attribute.");
             }
         }
 
@@ -707,14 +675,12 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
 
 
     /**
-     * Returns true if the byte is a separator as defined by V0 of the cookie
-     * spec.
+     * Returns true if the byte is a separator as defined by V0 of the cookie spec.
      */
     private static boolean isV0Separator(final char c) {
         if (c < 0x20 || c >= 0x7f) {
             if (c != 0x09) {
-                throw new IllegalArgumentException(
-                        "Control character in cookie value or attribute.");
+                throw new IllegalArgumentException("Control character in cookie value or attribute.");
             }
         }
 
@@ -723,17 +689,16 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
 
 
     /**
-     * Given a starting position after an initial quote character, this gets
-     * the position of the end quote. This escapes anything after a '\' char
-     * JVK RFC 2616
+     * Given a starting position after an initial quote character, this gets the position of the end quote. This escapes
+     * anything after a '\' char JVK RFC 2616
      */
-    private static int getQuotedValueEndPosition(byte bytes[], int off, int end){
+    private static int getQuotedValueEndPosition(byte bytes[], int off, int end) {
         int pos = off;
         while (pos < end) {
             if (bytes[pos] == '"') {
                 return pos;
             } else if (bytes[pos] == '\\' && pos < (end - 1)) {
-                pos+=2;
+                pos += 2;
             } else {
                 pos++;
             }
@@ -744,7 +709,7 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
 
 
     private static boolean equals(String s, byte b[], int start, int end) {
-        int blen = end-start;
+        int blen = end - start;
         if (b == null || blen != s.length()) {
             return false;
         }
@@ -759,26 +724,16 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
 
 
     /**
-     * Returns true if the byte is a whitespace character as
-     * defined in RFC2619
-     * JVK
+     * Returns true if the byte is a whitespace character as defined in RFC2619 JVK
      */
     private static boolean isWhiteSpace(final byte c) {
         // This switch statement is slightly slower
         // for my vm than the if statement.
         // Java(TM) 2 Runtime Environment, Standard Edition (build 1.5.0_07-164)
         /*
-        switch (c) {
-        case ' ':;
-        case '\t':;
-        case '\n':;
-        case '\r':;
-        case '\f':;
-            return true;
-        default:;
-            return false;
-        }
-        */
+         * switch (c) { case ' ':; case '\t':; case '\n':; case '\r':; case '\f':; return true; default:; return false;
+         * }
+         */
         if (c == ' ' || c == '\t' || c == '\n' || c == '\r' || c == '\f') {
             return true;
         } else {
@@ -810,12 +765,12 @@ public final class LegacyCookieProcessor extends CookieProcessorBase {
         int dest = 0;
 
         while (src < len) {
-            if (copy[src] == '\\' && src < len && copy[src+1]  == '"') {
+            if (copy[src] == '\\' && src < len && copy[src + 1] == '"') {
                 src++;
             }
             copy[dest] = copy[src];
-            dest ++;
-            src ++;
+            dest++;
+            src++;
         }
         bc.setBytes(copy, 0, dest);
     }
